import {AfterViewInit, Component, ElementRef, ViewChild} from '@angular/core';
import PhylocanvasGL, {TreeTypes} from "@phylocanvas/phylocanvas.gl";
import {Router} from '@angular/router';
import {DataService} from 'src/app/services/data.service';
import {MatDialog} from '@angular/material/dialog';
import {DialogComponent} from '../dialog/dialog.component';
import {MatSnackBar} from "@angular/material/snack-bar";
import {forkJoin, take} from "rxjs";

@Component({
  selector: 'slf-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.scss']
})
export class HomeComponent implements AfterViewInit {
  @ViewChild('demo') demoDiv!: ElementRef<HTMLDivElement>;
  private tree: PhylocanvasGL;

  constructor(private dataService: DataService,
              private router: Router,
              public dialog: MatDialog,
              private readonly snackBar: MatSnackBar) {
  }

  public ngAfterViewInit(): void {
    this.dataService.getNewick()
      .subscribe(newickTree => {
        this.renderizeTree(newickTree);
      });
  }

  private renderizeTree(newickTree: string): void {
    const screenSize = this.demoDiv.nativeElement.getBoundingClientRect();
    screenSize.width = screenSize.width * 0.95;
    screenSize.height = screenSize.height * 3;
    const metadata = 'id,colour\nRosaceae,red\nUlmaceae,red'; //Esto serian diferentes opciones de personalización de nodos en formato csv
    this.tree = new PhylocanvasGL(
      this.demoDiv.nativeElement,
      {
        interactive: false,
        showLabels: true,
        showLeafLabels: true,
        size: screenSize,
        alignLabels: true,
        nodeSize: 16,
        fillColour: '#3f51b5',
        source: newickTree,
        type: TreeTypes.Rectangular,
        styles: (
          metadata
            .split("\n")
            .map((line) => line.split(','))
            .reduce(
              (styles, [id, fillColour]) => {
                styles[id] = {fillColour};
                return styles;
              },
              {}
            )
        ),
      },
    );
    this.tree.setProps({
      interactive: true
    });
  }

  public onGoToFamily(): void {
    const param = this.tree['props']['selectedIds'];
    if (param !== undefined) {
      const familyId = param[0];

      if (familyId === 'AmaranthaceaeChenopodiaceae') {
        forkJoin(
          {
            // take(1) is needed as forkJoin requires the observable to complete
            Amaranthaceae: this.dataService.hasFamily('Amaranthaceae').pipe(take(1)),
            Chenopodiaceae: this.dataService.hasFamily('Chenopodiaceae').pipe(take(1))
          }
        ).subscribe(familiesPresence => {
          this.onOpenDialog(familiesPresence['Amaranthaceae'], familiesPresence['Chenopodiaceae']);
        });
      } else {
        this.dataService.hasFamily(familyId)
          .subscribe(hasFamily => {
            if (hasFamily) {
              this.router.navigate(['family', familyId]);
            } else {
              this.snackBar.open(`Data not found for family: ${param}`, 'Close', {
                horizontalPosition: "end",
                verticalPosition: "bottom",
                duration: 4000
              });
            }
          });
      }
    }
  }

  private onOpenDialog(existAmaranthaceae: boolean, existChenopodiaceae: boolean): void {
    const dialogRef = this.dialog.open(DialogComponent, {
      width: '450px',
      data: {'existAmaranthaceae': existAmaranthaceae, 'existChenopodiaceae': existChenopodiaceae}
    });
  }
}
